/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/cli/CliParameter.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.cli;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import com.netapp.common.cli.enumeration.CliEnumTransformer;
import com.netapp.common.cli.enumeration.EqualsCliEnumTransformer;
import com.netapp.common.cli.validator.ValidatorType;

/**
 * Annotates a method param or a setter method with the information necessary to transform
 * it into an option in a CLI.
 * @author sirianni
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
@Target({ElementType.PARAMETER, ElementType.METHOD})
@Retention(RetentionPolicy.RUNTIME)
public @interface CliParameter {
    
    String option();
    /**
     * Prefix to be used for argument names of children of this parameter
     * <b>REQUIRED to uniquely identify child arguments relative to other parameters</b> 
     */
    String prefix() default "";
    String argumentName() default "";
    Type type() default Type.PRIMITIVE;
    ValidatorType[] validators() default { };
    
    /**
     * For enumerations, this is used to transform the enumeration value
     * to a string cli option, and back
     */
    Class<? extends CliEnumTransformer> enumTransformer() default EqualsCliEnumTransformer.class;
    
    /**
     * ARRAY has been deprecated. Just use PRIMITIVE with an array. This will work for more
     * than just String []. This new use of arrays does not work well with boolean.
     * @author joshb
     *
     */
    public enum Type {
        PRIMITIVE, BEAN, @Deprecated ARRAY, @Deprecated ENUM
    }
}
