/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/cli/UsageHelpPrinter.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.cli;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Properties;
import java.util.Set;
import java.util.Map.Entry;

import com.netapp.common.logging.NALogger;

/**
 * @author gaddy
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class UsageHelpPrinter {

    private static NALogger logger = NALogger.getLogger(UsageHelpPrinter.class);

    private static final String HELP_FILE = "properties/help.properties";

    private Properties props;

    public UsageHelpPrinter(String productName) {
        loadHelpFile();
    }

    public String getUsage(String command) {
        
        try {
            StringWriter sw = new StringWriter();
            printUsage(new PrintWriter(sw), command.toLowerCase());
            return sw.toString();
        }
        // We don't want a bug in this class to cause an exception that the user see's
        // We will just return the empty string and let the code that called this realize we
        // could not provide any usage for the command and default back to the apache usage.
        // CHECKSTYLE:OFF
        catch (Throwable t) {
        // CHECKSTYLE:ON
            // FIXLOG: Externalize
            logger.internal("Could not get usage for command %s", t, command);
        }
        
        return "";
        
    }

    private void printUsage(PrintWriter pw, String command) {

        List<CommandSyntax> syntaxList = getSyntaxListForCommand(command);

        if (syntaxList.size() == 0) {
            return;
        }

        //pw.format("Usage syntax for \"%s %s\":\n", this.productName, command);
        for (CommandSyntax entry : syntaxList) {
            pw.println(entry);
        }
    }

    private List<CommandSyntax> getSyntaxListForCommand(String command) {

        List<CommandSyntax> syntaxList = new ArrayList<CommandSyntax>(10);

        Set<Entry<Object, Object>> propSet = this.props.entrySet();
        for (Entry<Object, Object> entry : propSet) {
            String name = (String) entry.getKey();

            String[] strs = name.split("\\.");
            if (strs.length != 3) {
                String msg = logger.internal(
                    "Invalid name in help file %s: %s. Name must be of form: 'command.subcommand.index'", HELP_FILE,
                    name);
                throw new RuntimeException(msg);
            }
            String nextCommand = strs[0];
            String nextSubcommand = strs[1];
            int index = Integer.parseInt(strs[2]);

            if (nextCommand.equals(command)) {
                String syntax = ((String) entry.getValue()).trim();

                CommandSyntax commandSyntax = new CommandSyntax(nextCommand, nextSubcommand, syntax, index);
                syntaxList.add(commandSyntax);
            }
        }

        Collections.sort(syntaxList);
        return syntaxList;

    }

    private void loadHelpFile() {

        ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
        this.props = new Properties();

        InputStream iStream = classLoader.getResourceAsStream(HELP_FILE);

        if (iStream != null) {
            try {
                this.props.load(iStream);
                iStream.close();
            }
            catch (IOException e) {
                // FIXLOG: Externalize
                String msg = logger.internal("Could not load properties from file '%s'", HELP_FILE);
                throw new RuntimeException(msg, e);
            }
        }
        else {
            // FIXLOG: Externalize
            String msg = logger.internal("Could not load properties from file '%s'", HELP_FILE);
            throw new RuntimeException(msg);
        }

    }

    class CommandSyntax implements Comparable<CommandSyntax> {

        private String command;
        private String subCommand;
        private String syntax;
        private int index;

        public CommandSyntax(String command, String subCommand, String syntax, int index) {
            this.command = command;
            this.subCommand = subCommand;
            this.syntax = syntax;
            this.index = index;
        }

        @Override
        public String toString() {
            return this.command + " " + this.subCommand + " " + this.syntax;
        }

        public int compareTo(CommandSyntax rhs) {
            if (this.index > rhs.index) {
                return 1;
            }
            else if (this.index < rhs.index) {
                return -1;
            }
            else {
                return 0;
            }
        }

    }

}
