/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/flow/ThreadpoolMBean.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.flow;

import java.util.Date;
import java.util.Set;

/**
 * Java Management Extensions (JMX) interface for the Threadpool
 * worker queues in the workflow engine.
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public interface ThreadpoolMBean {
    /**
     * Get the number of threads declared in this pool.
     * Due to concurrency issues, this may not be the same as
     * {@link #getActiveCount()} + {@link #getIdleCount()}.
     * 
     * @return Declared threadpool size
     */
    int getSize();
    
    /**
     * Get the list of operations this threadpool is configured
     * to run.
     * @return Set of operation names
     */
    Set<String> getValidOperations();
    
    /**
     * Get the number of threads currently executing an operation.
     * @return Active thread count
     */
    int getActiveCount();
    
    /**
     * Get the number of threads in the idle queue waiting for an operation.
     * @return Idle thread count
     */
    int getIdleCount();
    
    /**
     * Get the list of thread names in this pool.
     * @return List of threads
     */
    String[] getThreads();
    
    /**
     * Get the total number of times an operation was queued
     * and executed in this threadpool.
     * @return Operation start count
     */
    long getOperationStartCount();
    
    /**
     * Get the most recent operation start time in this pool.
     * @return Last operation start time
     */
    Date getLastOperationStartTime();
    
    /**
     * Get the most recent operation stop time in this pool.
     * An operation stop is recorded whenever an operation is paused
     * and returned to the work queue or ends in a terminal state.
     * @return Last operation stop time.
     */
    Date getLastOperationStopTime();
}
