/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/flow/soap/handlers/binding/WebServiceWorkflowBindingHandler.java#1 $
 * 
 * Copyright (c) 2009 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.flow.soap.handlers.binding;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.xml.ws.WebServiceContext;

import org.apache.cxf.transport.http.AbstractHTTPDestination;

import com.netapp.common.MsgKey;
import com.netapp.common.flow.FlowException;
import com.netapp.common.flow.soap.WorkflowBindingHandler;
import com.netapp.common.logging.NALogger;
import com.netapp.common.util.StringUtils;

/**
 * An external workflow binding handler that retrieves the data
 * from the HTTP request object. This class depends on WebServiceContext
 * from CXF. The WebServiceContext object keeps the HttpServletRequest in 
 * a ThreadLocal variable so that we can access it well away from the actual
 * service layer.
 * 
 * To use this handler, the type must be specified as <b>WS-HTTP</b>.
 * 
 * @author josh
 *
 */
public class WebServiceWorkflowBindingHandler implements WorkflowBindingHandler {
    private static final NALogger LOGGER = NALogger.getLogger(WebServiceWorkflowBindingHandler.class);
    
    @Resource
    private WebServiceContext context;

    /**
     * This handler can process type <b>WS-HTTP</b>. The check is actually
     * case insensitive.
     */
    public boolean canHandleType(String type) {
        if (type.equalsIgnoreCase("ws-http")) {
            return true;
        }
        
        return false;
    }

    public String getValue(String name) {
        HttpServletRequest request = null;

        if (null != getContext() && null != getContext().getMessageContext()) {
            request = (HttpServletRequest) getContext().getMessageContext().get(AbstractHTTPDestination.HTTP_REQUEST);
        }
        
        return getRequestHeaderValue(name, request);
    }
    
    private String getRequestHeaderValue(String header, HttpServletRequest request) {            
        if (StringUtils.isBlank(header)) {
            throw new FlowException(LOGGER.error(MsgKey.FLOW_SOAP_SERVICE_MISSING_NAME));
        }
        
        if (null == request) {
            LOGGER.warn(MsgKey.FLOW_SOAP_SERVICE_NO_HTTP_REQUEST, header);
            return null;
        }
        
        String value = null;

        if (header.equalsIgnoreCase("remote_addr")) {
            value = request.getRemoteAddr();
        }
        else if (header.equalsIgnoreCase("remote_host")) {
            value = request.getRemoteHost();
        }
        else {
            value = request.getHeader(header);
        }

        return value;
    }

    public WebServiceContext getContext() {
        return context;
    }

    public void setContext(WebServiceContext context) {
        this.context = context;
    }

}
