/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/linux/RH4MultipathHandler.java#1 $
 * 
 * Copyright (c) 2008 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform.linux;

import java.io.File;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * This handler manages block devices from device-mapper-multipath in RHEL 4.
 * This differs from {@link RH5MultipathHandler} in that
 * the partition naming is somewhat inconsistent and broken.
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class RH4MultipathHandler extends AbstractDeviceMapperMultipathHandler {
    /**
     * Matches "(/dev/mapper/LUN)(p1)" The device name has to be reluctant.
     * This is terrible because multipath isn't consistent. If the last character of the LUN name is numeric,
     * the name is suffixed with 'pX' where X is the partition number.
     * If the last character is non-numeric, they just append 'X'.
     */
    private static final Pattern MPIO_PATTERN = Pattern.compile("(/dev/(mapper|mpath)/\\S+?)(?:p?(\\d+))?");
    private static final int MPIO_DISK_GROUP = 1;
    private static final int MPIO_PARTITION_GROUP = 3;
    
    /**
     * Matches /dev/mapper devices that end in a number, so we know to append pX instead of just X.
     */
    private static final Pattern MPIO_NUMERIC_DEVICE_PATTERN = Pattern.compile("/dev/(mapper|mpath)/.*?(\\d)?");
    private static final int MPIO_NUMERIC_PARTITION_GROUP = 2;
    
    public RH4MultipathHandler(OperatingSystemImpl operatingSystem) {
        super(operatingSystem);
    }

    public File deviceForPartition(File disk, String partition) {
        Matcher matcher = MPIO_NUMERIC_DEVICE_PATTERN.matcher(disk.getAbsolutePath());
        if (matcher.matches()) {
            String fileName = disk.getAbsolutePath().replace("mpath", "mapper");
            return new File(fileName + (matcher.group(MPIO_NUMERIC_PARTITION_GROUP) != null ? "p" : "") + partition);
        }
        return null;
    }

    public String extractFullDiskPathElement(File file) {
        Matcher matcher = MPIO_PATTERN.matcher(file.getAbsolutePath().replace("mpath", "mapper"));
        if (matcher.matches()) {
            return matcher.group(MPIO_DISK_GROUP);
        }
        return null;
    }

    public String extractPartitionPathElement(File file) {
        Matcher matcher = MPIO_PATTERN.matcher(file.getAbsolutePath());
        if (matcher.matches()) {
            return matcher.group(MPIO_PARTITION_GROUP);
        }
        return null;
    }
}
