/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/linux/ScsiDeviceHandler.java#1 $
 * 
 * Copyright (c) 2008 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform.linux;

import java.io.File;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.netapp.common.execution.ShellCommand;
import com.netapp.common.execution.ShellResult;
import com.netapp.common.platform.DeviceType;

/**
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class ScsiDeviceHandler extends AbstractBlockDeviceHandler {
    private static final Pattern SCSI_DEVICE_PATTERN = Pattern.compile("(/dev/sd[a-z]+)(\\d+)?");
    private static final int SCSI_DEVICE_DISK_GROUP = 1;
    private static final int SCSI_DEVICE_PARTITION_GROUP = 2;
    
    /**
     * Matches "brw-rw----    1 root     disk      65, 153 Jun 24  2004 /dev/sdz9"
     */
    private static final Pattern LS_ENTRY_PATTERN = Pattern.compile(".*?(\\d+),\\s+(\\d+).*?(/dev/sd.+).*?");

    
    public ScsiDeviceHandler(OperatingSystemImpl operatingSystem) {
        super(operatingSystem);
    }
    
    public String driverName() {
        return "sd";
    }
    
    /**
     * @see com.netapp.common.platform.linux.BlockDeviceHandler#determineDeviceType(java.io.File)
     */
    public DeviceType determineDeviceType(File file) {
        Matcher matcher = SCSI_DEVICE_PATTERN.matcher(file.getAbsolutePath());
        if (matcher.matches()) {
            if (matcher.group(SCSI_DEVICE_PARTITION_GROUP) != null) {
                return DeviceType.PARTITION;
            }
            return DeviceType.BASE_DEVICE;
        }
        return DeviceType.NON_DEVICE;
    }

    /**
     * @see com.netapp.common.platform.linux.BlockDeviceHandler#deviceForPartition(java.io.File, java.lang.String)
     */
    public File deviceForPartition(File disk, String partition) {
        Matcher matcher = SCSI_DEVICE_PATTERN.matcher(disk.getAbsolutePath());
        if (matcher.matches()) {
            return new File(matcher.group(SCSI_DEVICE_DISK_GROUP) + partition);
        }
        return null;
    }

    /**
     * @see com.netapp.common.platform.linux.BlockDeviceHandler#deviceFromMajorMinor(int, int)
     */
    public File deviceFromMajorMinor(int major, int minor) {
        // Note - Do this with stat() and jni -jrs
        ShellResult lsResult = getShellExecutor().execute(new ShellCommand("ls", "-l", "/dev/sd*"));
        final List<String> stdOutput = lsResult.getStdOut();
        for (String lsEntry : stdOutput) {
            Matcher lsMatcher = LS_ENTRY_PATTERN.matcher(lsEntry);
            if (lsMatcher.matches()) {
                int deviceMajorNumber = Integer.parseInt(lsMatcher.group(1));  
                int deviceMinorNumber = Integer.parseInt(lsMatcher.group(2));
                if (deviceMajorNumber ==  major && deviceMinorNumber == minor) {
                    return new File(lsMatcher.group(3));
                }
            }
        }
        return null;
    }

    /**
     * @see com.netapp.common.platform.linux.BlockDeviceHandler#extractFullDiskPathElement(java.io.File)
     */
    public String extractFullDiskPathElement(File file) {
        Matcher matcher = SCSI_DEVICE_PATTERN.matcher(file.getAbsolutePath());
        if (matcher.matches()) {
            return matcher.group(SCSI_DEVICE_DISK_GROUP);
        }
        return null;
    }

    /**
     * @see com.netapp.common.platform.linux.BlockDeviceHandler#extractPartitionPathElement(java.io.File)
     */
    public String extractPartitionPathElement(File file) {
        Matcher matcher = SCSI_DEVICE_PATTERN.matcher(file.getAbsolutePath());
        if (matcher.matches()) {
            return matcher.group(SCSI_DEVICE_PARTITION_GROUP);
        }
        return null;
    }
}