/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/remote/jvm/ShellExecutorJvmLauncher.java#1 $
 * 
 * Copyright (c) 2008 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.remote.jvm;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import com.netapp.common.MsgKey;
import com.netapp.common.execution.ShellCommand;
import com.netapp.common.execution.ShellExecutor;
import com.netapp.common.execution.ShellResult;
import com.netapp.common.logging.NALogger;

/**
 * @author claird
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public abstract class ShellExecutorJvmLauncher implements JvmLauncher {

    private static final NALogger logger = NALogger.getLogger();
    
    private ShellExecutor shellExecutor;
    private File javaLauncher;

    ShellExecutorJvmLauncher() {
        super();
    }
    
    ShellExecutorJvmLauncher(ShellExecutor shellExecutor, File javaLauncher) {
        this.shellExecutor = shellExecutor;
        this.javaLauncher = javaLauncher;
    }

    /**
     * @see com.netapp.common.remote.jvm.JvmLauncher#launch(com.netapp.common.remote.jvm.JvmLaunchCommand)
     */
    public JvmLaunchResult launch(JvmLaunchCommand jvmLaunchCommand) { 
        /*
         * Intentionally not using absolutePath here because getAbsolutePath() resolves
         * against the current process working directory, but the ShellExecutor may have 
         * intentionally set a different working directory, or it may be relying on PATH 
         * to resolve the javaLauncher
         */
        List<String> commandList = new ArrayList<String>();
        commandList.add(javaLauncher.getPath());
        commandList.add(jvmLaunchCommand.getClassName());
        commandList.addAll(Arrays.asList(jvmLaunchCommand.getArgs()));
    
        ShellCommand shellCommand = new ShellCommand(commandList);
        
        ShellResult shellResult = new ShellResult(this.shellExecutor.executeInteractive(shellCommand));
        ShellJvmLaunchResult shellJvmLaunchResult = new ShellJvmLaunchResult(shellResult);
        if (shellJvmLaunchResult.isRunning()) {
            logger.debug(MsgKey.EXECUTION_JVM_RESULT_STILL_RUNNING, jvmLaunchCommand.getClassName(), shellJvmLaunchResult);
        }
        else {
            logger.debug(MsgKey.EXECUTION_JVM_RESULT, jvmLaunchCommand.getClassName(), shellJvmLaunchResult.getExitValue(), 
                shellJvmLaunchResult);
        }
        
        return shellJvmLaunchResult;
    }

    public ShellExecutor getShellExecutor() {
        return this.shellExecutor;
    }

    public void setShellExecutor(ShellExecutor shellExecutor) {
        this.shellExecutor = shellExecutor;
    }

}