/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/server/JettyServer.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.server;

import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import com.netapp.common.MsgKey;

import com.netapp.common.logging.NALogger;
import com.netapp.common.util.ConfigBundle;


/**
 * Start a Jetty server using a xbean definition. The configuration file is the path
 * to the xbean definition. Inside of the definition, this implementation will look for 
 * a bean named <b>jetty.server</b> by default. This default name can be overridden
 * with a call to <b>setBeanName</b> or by using the overriden constructor to set it.
 * 
 * @author joshb
 *
 */
public class JettyServer extends AbstractJettyServer {
    private static final NALogger LOGGER = NALogger.getLogger(JettyServer.class);
    
    public static final String[] DEFAULT_CONFIG_FILES = { "jetty.default", "jetty.config", "jetty.override" };
    
    private String configurationFile;
    
    public JettyServer(String configurationFile) {
        this (configurationFile, DEFAULT_BEAN_NAME);
    }
    
    public JettyServer(String configurationFile, String beanName) {
        setConfigurationFile(configurationFile);
        setBeanName(beanName);
        setServerName(DEFAULT_SERVER_NAME);
    }

    public String getConfigurationFile() {
        return configurationFile;
    }

    public void setConfigurationFile(String configurationFile) {
        this.configurationFile = configurationFile;
    }

    /**
     * Load the jetty server and config bundle components.
     * This implementation uses a spring ApplicationContext.
     */
    @Override
    protected void loadComponents() {
        LOGGER.debug(MsgKey.SERVER_JETTY_LOADING_CONFIGURATION, getConfigurationFile());
        ApplicationContext context = new ClassPathXmlApplicationContext(getConfigurationFile());
        
        LOGGER.debug(MsgKey.SERVER_JETTY_LOADING_SERVER_INSTANCE, getBeanName());
        setServer((org.mortbay.jetty.Server) context.getBean(getBeanName()));
        
        ConfigBundle bundle;
        if (context.containsBean(DEFAULT_CONFIG_BUNDLE_NAME)) {
            bundle = (ConfigBundle)context.getBean(DEFAULT_CONFIG_BUNDLE_NAME);
        }
        else {
            bundle = new ConfigBundle(DEFAULT_CONFIG_FILES);
        }
        setConfigBundle(bundle);
    }
}
