/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/util/CacheMBean.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.util;

import java.util.Date;

/**
 * Management interface for {@link Cache}.
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public interface CacheMBean {
    /**
     * Retrieve a list of the current keys in the cache.
     * @return Current known keys
     */
    String[] getKeys();
    
    /**
     * Get the current size of the cache. This may include references that have been
     * garbage collected but not scavenged yet.
     * @return Current cache size
     */
    int getCurrentCount();
    
    /**
     * Get the number of items that have ever been added to the cache.
     * @return Number of cache puts
     */
    long getLifetimeCount();
    
    /**
     * Get the number of times {@link Cache#get} returned a non-null value
     * @return Number of successful cache hits
     */
    long getHitCount();
    
    /**
     * Get the number of times {@link Cache#get} returned a null value
     * @return Number of cache misses
     */
    long getMissCount();
    
    /**
     * Get the last time this cache was scavenged.
     * @return Last scavenge time
     */
    Date getLastScavengeTime();
    
    /**
     * Reset the JMX statistics counters for this cache.
     *
     */
    void resetCounters();
    
    /**
     * Force a cache scavenge to remove expired objects.
     *
     */
    void scavenge();
    
    /**
     * Eject all items from the cache.
     *
     */
    void clear();
}
