/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/zapi/generator/autozapi/AutoZapiErrdefGenerator.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.zapi.generator.autozapi;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;

import com.netapp.common.generator.NetappClassGenerator;
import com.netapp.common.util.StringUtils;
import com.netapp.common.zapi.ErrorCode;
import com.netapp.common.zapi.parser.Errdef;

/**
 * Generate an enumeration of ZAPI response codes
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class AutoZapiErrdefGenerator extends NetappClassGenerator {
    private Collection<Errdef> errdefs;
    public AutoZapiErrdefGenerator(String packageName, Collection<Errdef> errdefs) {
        super(packageName, AutoZapiGenerator.FORMATTER);
        this.errdefs = errdefs;
        
        generateClassFile();
    }
    
    protected Collection<Errdef> getErrdefs() {
        return this.errdefs;
    }
    
    protected void generateClassFile() {
        addImport(ErrorCode.class.getCanonicalName());
        addLine("public enum ErrorEnum implements ErrorCode {");
        increaseIndent();
        
        Iterator<Errdef> iter = this.errdefs.iterator();
        while (iter.hasNext()) {
            Errdef def = iter.next();
            if (!StringUtils.isEmpty(def.getDesc())) {
                addJavadocLine(def.getDesc());
                addLine(def.getName() + "(" + def.getCode() + ")" + (iter.hasNext() ? "," : ";"));
            }
        }
        addLine("private final int code;");
        addLine("private ErrorEnum(int code) { this.code = code; }");
        addLine("public int getCode() { return this.code; }");
        addLine("public boolean isCode(int code) { return this.code == code; }");
        
        decreaseIndent();
        addLine("}");
    }

    public void writeCode(File dir) throws IOException {
        this.writeCode(dir.getAbsolutePath(), "ErrorEnum.java");
    }
}
