/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/zapi/parser/Namespace.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.zapi.parser;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import com.netapp.common.MsgKey;
import com.netapp.common.logging.NALogger;
import com.netapp.common.zapi.generator.GeneratorFactory;

/**
 * ZAPI namespace containing {@link com.netapp.common.zapi.parser.Api}
 * and {@link com.netapp.common.zapi.parser.Typedef} classes.
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class Namespace {
    private static final NALogger logger = NALogger.getLogger(Namespace.class);

    /**
     * The zapi name of this namespace ("filer", "com-e", "dfm", etc)
     */
    private String name;
    
    /**
     * The defined {@Link Type types} defined on this namespace, keyed by the type's zapi name.
     */
    private Map<String, Type> types = new TreeMap<String, Type>();

    private final ZapidocParser parser;
    
    /**
     * Construct a new Namespace. There should only be one Namespace in a given
     * generator context per name. Use {@link #getNamespace(String)} instead
     * to retrieve the existing namespace or construct a new one.
     * @param name
     */
    Namespace(ZapidocParser parser, String name) {
        this.parser = parser;
        this.name = name;
    }
    
    /**
     * Retrieve the declared zapi name of this namespace
     * @return zapi name
     */
    public String getName() {
        return this.name;
    }
    
    /**
     * Add a new {@link Type} to this namespace. It is an error to attempt to add
     * a second type with the same name as an existing type.
     * 
     * @param type new type to add
     */
    
    public void addType(Type type) {
        if (types.containsKey(type.getName())) {
            this.parser.appendError(MsgKey.ZAPI_GENERATOR_REDEFINE_TYPE, type.getName(), this.name);
        }
        else {
            types.put(type.getName(), type);
        }
    }
    
    /**
     * Return a collection of all defined types on this namespace
     * @return collection of defined types
     */
    public Collection<Type> getTypes() {
        return this.types.values();
    }
    
    /**
     * Lookup and return the type in this namespace for the supplied 
     * @param name
     * @return Type for the supplied name
     */
    public Type getType(String name) {
        Type type = this.types.get(name);
        if (type == null) {
            type = SimpleType.SIMPLE_TYPES.get(name);
        }
        return type;
    }
    
    /**
     * Return a collection of all defined types on this namespace
     * @return collection of defined types
     */
    public Collection<Api> getApis() {
        List<Api> apis = new LinkedList<Api>();
        
        for (Type type : this.types.values()) {
            if (type instanceof Api) {
                apis.add((Api) type);
            }
        }
        
        return apis;
    }
    
    public void addWordReplacements(GeneratorFactory<?, ?, ?, ?> generatorFactory) {
        
        for (Type type : getTypes()) {
            type.addWordReplacements(generatorFactory);
        }
    }
    
    /**
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return this.getName();
    }
    
}
